#Licensed Materials - Property of IBM 
#Rational ClearQuest 
#Copyright IBM Corp. 2008 All Rights Reserved 

#!/usr/bin/perl
use Config;
use Encode;
use Pod::Usage;
use POSIX qw(locale_h);

BEGIN {

	if($Config{osname} eq "MSWin32"){
		$ENV{PATH} = $ENV{CQCLI_PATH};
		$ENV{CLASSPATH} = $ENV{CQCLI_CLASSPATH};
		$ENV{'PERL_INLINE_JAVA_JNI'} = 1;
	}
	
	my $locale = setlocale(LC_CTYPE);
	if(($locale =~ /zh/i) || ($locale =~ /ch/i) ) {
		$ENV{CQCLI_LOCALE}	= "zh";
	}elsif(($locale =~ /ja/i) || ($locale =~ /jap/i)){
		$ENV{CQCLI_LOCALE}	= "ja";
	}
	
	if(scalar(@ARGV)== 0)
	{
		my $exit_status = 'NOEXIT';
		my $podfile = "cqtool.pod";
		my $mandir = $ENV{CQCLI_MAN_DIR};
		pod2usage(-verbose=>3,-exitval => $exit_status ,-input=>$podfile,-pathlist=>$mandir);
		exit 0;
	}
}


#use warnings;
#use strict;
use Term::ReadLine;
use Text::ParseWords;
use cqcli::Util;
use cqcli::commonservice;
use cqcli::controller;

BEGIN {

	my $loc = localeinfo();
	
	if(($loc =~ /CN/i) || ($loc =~ /cn/i) ) {
		$ENV{CQCLI_LOCALE}	= "zh";
	}
	elsif(($loc =~ /JP/i) || ($loc =~ /jp/i)){
		$ENV{CQCLI_LOCALE}	= "jp";
	}
	elsif(($loc =~ /DE/i) || ($loc =~ /de/i)){
		$ENV{CQCLI_LOCALE}	= "de";
	}
	elsif(($loc =~ /ES/i) || ($loc =~ /es/i)){
		$ENV{CQCLI_LOCALE}	= "es";
	}
	elsif(($loc =~ /FR/i) || ($loc =~ /fr/i)){
		$ENV{CQCLI_LOCALE}	= "fr";
	}
	elsif(($loc =~ /KR/i) || ($loc =~ /kr/i)){
		$ENV{CQCLI_LOCALE}	= "kr";
	}
	elsif(($loc =~ /IT/i) || ($loc =~ /it/i)){
		$ENV{CQCLI_LOCALE}	= "it";
	}
	elsif(($loc =~ /BR/i) || ($loc =~ /br/i)){
		$ENV{CQCLI_LOCALE}	= "br";
	}
	elsif(($loc =~ /US/i) || ($loc =~ /us/i)){
		$ENV{CQCLI_LOCALE}	= "en";
	}

}

$SIG {"INT"} = \&ctrl_c_handler;
$SIG{'QUIT'} = \&ctrl_c_handler;

$Controller = GetCQCLIController();


sub cqtool_interactive
{
	my $input = shift;
	SetCQCLIInteractiveMode(1);
	return ($Controller->execute($input));

}


sub  cqtool_noninteractive
{
	my $input = shift;
	SetCQCLIInteractiveMode(undef);
	return ($Controller->execute($input));

}

sub Interactive_Prompt()
{
	my $this;
	my $input;
	my $bracing = 0;
	my $term = new Term::ReadLine "CQTOOL"; 
	# my $OUT = $term->OUT || \*STDOUT;
	$|=0;
	
	while(1) {
		$this = $term->readline("cqtool> ");
		$input = $this;
		$input = cqclitrim($input);
		
	if($Config{osname} eq "MSWin32"){
		#This is required to preserve character encoding from perl to java on operating systems
		if($ENV{CQCLI_LOCALE} eq 'en') {
			$input  = decode('cp1252',$input);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'zh') {
			$input  = decode('cp936',$input);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'jp') {
			$input  = decode('cp932',$input);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'kr') {
			$input  = decode('cp949',$input);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'de') {
			$input  = decode('cp1252',$input);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'br') {
			$input  = decode('cp1252',$input);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'it') {
			$input  = decode('cp1252',$input);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'fr') {
			$input  = decode('cp1252',$input);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'es') {
			$input  = decode('cp1252',$input);
		}
	}
	else {
		$input = decode('utf8',$input);
	}



		
		if ($input =~ /\S/) {
			$term->addhistory($input);
		}
		
		if($input =~ s/^!\s*//) {
			system $input;
		}
		elsif ($input =~ s/^\?\s*//) {
			if($input =~ /^([A-Z]|perl)/) {
				system "perldoc" , $input;
			}
			else {
				system "perldoc" , "-f" ,$input;
			}
		}elsif($input eq '')
		{
			next;
		}
		else {
			$Controller->execute($input);
		}
		$input = "";
	}
}


#MAIN program starts here

if( scalar(@ARGV) > 0)
{
	my $cmd  = $ARGV[0];
	my  $cmdName = lc $cmd;
	if($cmdName eq "login")
	{
		my $cliargv = join ' ' , @ARGV;
		cqclitrim($cliargv);
		$ret = undef;
		$ret = cqtool_interactive($cliargv);
		if( $ret && GetSession() ){
			Interactive_Prompt(); 
		}else {
			exit -1;
		}
	}
	elsif($cmdName =~ /^-h(elp)?$/i)
	{
		my $exit_status = 'NOEXIT';
		my $podfile = "cqtool.pod";
		my $mandir = $ENV{CQCLI_MAN_DIR};
		pod2usage(-verbose=>1,-exitval => $exit_status ,-input=>$podfile,-pathlist=>$mandir);
		exit 0;
	}elsif($cmdName =~ /^-m(an)?$/i)
	{
		my $exit_status = 'NOEXIT';
		my $podfile = "cqtool.pod";;
		my $mandir = $ENV{CQCLI_MAN_DIR};
		pod2usage(-verbose=>3,-exitval => $exit_status ,-input=>$podfile,-pathlist=>$mandir);
		exit 0;
	}elsif($cmdName =~ /^run_script/i)
	{	
		my $cliargv = join ' ' , @ARGV;
		cqclitrim($cliargv);
		my @filename = undef;
		$filename[0] = $ARGV[1];
		$filename[1] = $cmd;
		$ret = $Controller->run_script(@filename);
		exit($ret);
	}
	else{
		#my $cliargv = join ' ' , @ARGV;
		my $cliargv = "";
		my $token = "";
		my $count=0;
		my $sq = false;

		if($Config{osname} eq "MSWin32"){
		
			for($count=0;$count<scalar(@ARGV);$count++){
			
				if($ARGV[$count] =~ /^(')(.+)/i){
					$sq = true;
					$token = $2 . " ";

			       if(($ARGV[$count] =~ /'$/i)){
					$sq = false;
					$token = $ARGV[$count];
					$token = substr($token,1,-1);
					goto FINISH;
				   }
					$count++;
					while($count<scalar(@ARGV)){
						if(($ARGV[$count] =~ /'$/i)) {
							$token = $token . $ARGV[$count];
							$sq = false;
							$token = substr($token,0,-1);
							goto FINISH;
						}
						else {
							$token = $token . $ARGV[$count] . " ";
						}
						$count++;
					}

					if($sq eq true){
						print "Can't find string terminator \"'\" anywhere before the end of command";exit 0;
					}
				
				}
				else {
					$token = $ARGV[$count];goto FINISH;
				}


				FINISH:$cliargv = $cliargv . $token . "#";
			}

			$cliargv = substr($cliargv,0,-1);
			cqclitrim($cliargv);
			chomp($cliargv);
		}
		else {
			$cliargv = join ' ' , @ARGV;
			cqclitrim($cliargv);
			chomp($cliargv);
		}
		
	if($Config{osname} eq "MSWin32"){
		if($ENV{CQCLI_LOCALE} eq 'zh') {
			$cliargv  = decode('cp936',$cliargv);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'jp') {
			$cliargv  = decode('cp932',$cliargv);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'kr') {
			$cliargv  = decode('cp949',$cliargv);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'de') {
			$cliargv  = decode('cp1252',$cliargv);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'br') {
			$cliargv  = decode('cp1252',$cliargv);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'it') {
			$cliargv  = decode('cp1252',$cliargv);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'fr') {
			$cliargv  = decode('cp1252',$cliargv);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'es') {
			$cliargv  = decode('cp1252',$cliargv);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'en') {
			$cliargv  = decode('cp1252',$cliargv);
		}
	}
	else {
		$cliargv  = decode('utf8',$cliargv);
	}
		
		$ENV{'PERL_INLINE_JAVA_JNI'} = 1;
		$ret = cqtool_noninteractive($cliargv);	
		exit($ret);
	}
}
elsif(scalar(@ARGV)== 0)
{
	my $exit_status = 'NOEXIT';
	my $podfile = "cqtool.pod";;
	my $mandir = $ENV{CQCLI_MAN_DIR};
	pod2usage(-verbose=>3,-exitval => $exit_status ,-input=>$podfile,-pathlist=>$mandir);
	exit 0;
}






##########################################################



