package cqcli::Util;

use 5.008006;
use strict;
use warnings;

require Exporter;


use Error qw(:try);
use Config;
use cqcli::Exception;
use cqcli::IOHandler;
use Term::ReadKey;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use cqcli::api::cqcliutility ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(cqclitrim cqcliltrim cqclirtrim GetLocalizedMessage GetStackTrace 	 
				CLIPrintError CLIPrintOutput CLIPrint_help_page CLIPrint_man_page CLIPrintIdOutput 	CLIPrintIdError 
				ThrowJavaException ThrowException ThrowParseException ThrowSyntaxErrorException ThrowValidationException ThrowUsageException 	
				AskForConnectionName AskForDBName AskForPassword AskForUserName AskForActionName AskForRecordId AskForYN AskForDuplicateRecordId
				ctrl_c_handler WantToExit AskForFileName CliJavaPrintOutput localeinfo  );

our $VERSION = '0.01';

our $InlineLibdir = undef;
BEGIN {

	my $pkgdir = undef;
	my $archinlinedir = undef;
	my $inlinedir = undef;
	$pkgdir = __FILE__;
	$pkgdir =~ s/\/Util\.pm//i;
	$pkgdir =~ s/\\Util\.pm//i;
	if($Config{osname} eq "MSWin32"){
		$pkgdir =~ s/\//\\/g;
		$archinlinedir = "inlineclass"."\\".$Config{archname};
		$inlinedir = $pkgdir."\\".$archinlinedir;
	}else{
		$pkgdir =~ s/\\/\//g;
		$archinlinedir = "inlineclass"."/".$Config{archname};
		$inlinedir = $pkgdir."/".$archinlinedir;
	}
	
	if(!(-d $inlinedir)){
		die "$! $inlinedir : please create the directory manually\n"
	}
	$ENV{PERL_INLINE_JAVA_DIRECTORY} = $inlinedir;
}

use Inline (

Java => <<'END_OF_JAVA_CODE' ,


import com.ibm.rational.CQTool.Utility.CQCLIJavaUtility;
import java.util.Locale;

public class CQCLIUtil
{
	private CQCLIUtil() { }
	public static String  getLocalizedMessage(String msgId, String[] ReplaceStrings)
	{
		return CQCLIJavaUtility.getLocalizedMessaged(msgId,ReplaceStrings);
	}
	
	public static String getStackTrace(Exception e )
	{
		return CQCLIJavaUtility.getStackTrace(e);
	}
	
		
	public static void printString(String message) throws Exception {
		System.out.print(message);
	}
	
	public static String getmylocale() {
		Locale locale = Locale.getDefault();
		return locale.getCountry();
	}

}

END_OF_JAVA_CODE
AUTOSTUDY => 1,
DIRECTORY => $ENV{PERL_INLINE_JAVA_DIRECTORY},

);


use Inline::Java qw(caught cast coerce);
# Preloaded methods go here.
#helper methods

sub GetLocalizedMessage
{
	my ($msgId,@Replace) = @_;
	my $gvtMsg =undef;
	return $gvtMsg unless defined($msgId);
	eval { $gvtMsg = cqcli::Util::CQCLIUtil->getLocalizedMessage($msgId,[@Replace]);};
	
	if ($@)
	{
		my $tmpexp = undef;
		my $ex = undef;
		my $gvtmsg = undef;
		my $jmsg = undef;
		my $jtrace = undef;
		if (caught("java.lang.Exception"))
		{
			  $tmpexp = $@;	  
			  $@ = undef;
			  $ex = new cqcli::JavaException("Localization of messages failed ");
			 
			  $jmsg = $tmpexp->getMessage();
			  $jtrace = GetStackTrace($tmpexp); 
			  $ex->appendJavaMessage($jmsg);
			  $ex->setJavaStackTrace($jtrace);
			  throw $ex;
		 }
		 else
		 {
			 $tmpexp = $@;
			 $@ = undef;
			 my $error = "Localization of messages failed"."\n".$tmpexp."\n";
			 $ex = new cqcli::Exception($error);
			 throw $ex;
		 }
	}

	return $gvtMsg;
}

sub localeinfo
{
	my $locale;
	eval { $locale = cqcli::Util::CQCLIUtil->getmylocale();};
	return $locale;
}

sub GetStackTrace
{
	my $ex = $_[0];
	my $stacktrace = undef;
	eval { $stacktrace = cqcli::Util::CQCLIUtil->getStackTrace($ex);};
	return $stacktrace;
}


sub CLIPrint_help_page
{
	my $cmdname = shift ;
	my $msg1 = GetLocalizedMessage('CQCLI_help_common',$cmdname);
	my $msg_id = "CQCLI_".$cmdname."_help";
	my $msg2 = GetLocalizedMessage($msg_id);
	CLIPrintOutput($msg1);
	CLIPrintOutput($msg2);
}


sub CLIPrint_man_page
{
	my $cmdname = shift ;
	my $msg_id = "CQCLI_".$cmdname."_man";
	my $msg1 = GetLocalizedMessage('CQCLI_man_common',$cmdname);
	my $msg2 = GetLocalizedMessage($msg_id);
	CLIPrintOutput($msg1);
	CLIPrintOutput($msg2);
}


sub CLIPrintOutput($)
{
	my $msg = shift;
	#CLIprintoutput($msg);
	CliJavaPrintOutput($msg);
}

sub CLIPrintError($)
{
	my $msg = shift;
	#CLIprinterror($msg);
	CliJavaPrintOutput($msg);
}

sub CLIPrintIdOutput
{
	my ($msgid,@msglist) = @_;
	my $msg = GetLocalizedMessage($msgid,@msglist);
	#CLIprintoutput($msg);
	CliJavaPrintOutput($msg);
}

sub CLIPrintIdError
{
	my ($msgid,@msglist) = @_;
	my $msg = GetLocalizedMessage($msgid,@msglist);
	#CLIprinterror($msg);
	CliJavaPrintOutput($msg);
}



sub ThrowException
{
	my ($msgid,@Replace) = @_;
	my  $msg = GetLocalizedMessage($msgid,@Replace);
	my $perlex = new cqcli::Exception($msg);
	$@=undef;
	$perlex->setErrorMessage($msg);
	throw $perlex;
    
}

sub ThrowJavaException
{
	my ($javaex,$msgid,@Replace) = @_;
	my  $msg = GetLocalizedMessage($msgid,@Replace);
	my $perlex = new cqcli::JavaException($msg);
	my $jmsg = $javaex->getMessage();
	if(!($jmsg)) { $jmsg = "null pointer exception"; }
	my $jtrace = GetStackTrace($javaex); 
	$perlex->appendJavaMessage($jmsg);
	$perlex->setJavaStackTrace($jtrace);
	$@=undef;
	$perlex->setErrorMessage($msg);
	$perlex->setJavaErrorMessage('('.$jmsg.')');
	throw $perlex;
    
}

sub ThrowSyntaxErrorException
{
	my ($msgid,@msgArray) = @_;
	my $msg = GetLocalizedMessage($msgid,@msgArray);
	my $perlex = new cqcli::SyntaxErrorException($msg);
	$perlex->setErrorMessage($msg);
	throw $perlex;
}

sub ThrowParseException
{
	my ($msgid,@msgArray) = @_;
	my $msg = GetLocalizedMessage($msgid,@msgArray);
	my $perlex =  new cqcli::ParseException($msg);
	$perlex->setErrorMessage($msg);
	throw $perlex;
}

sub ThrowValidationException
{
	my ($msgid,@msgArray) = @_;
	my $msg = GetLocalizedMessage($msgid,@msgArray);
	my $perlex =  new cqcli::ValidationException($msg);
	$perlex->setErrorMessage($msg);
	throw $perlex;
}


sub ThrowUsageException
{

	my ($cmd,$cmdopt) = @_;
	my $perlex = new cqcli::UsageException($cmd,$cmdopt);
	$perlex->SetCmdInfo($cmd,$cmdopt);
	throw $perlex;
}






sub AskForUserName
{
	my $cnt = $_[0];
	if( !($cnt) ){
		$cnt = 3;
	}
	my $i = 0;
	my $promptmsg = GetLocalizedMessage('CQCLI_Prompt_Username');
	my $repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_Username');
	my $username = undef;
	while($i<$cnt)
	{		
		CLIPrintOutput($promptmsg);
		$username=<STDIN>;
		if(!($username)){
			return ;
		}
		chomp($username);
		$username=cqclitrim($username);
		if(length $username){
			return $username;
		}
		else{
			$promptmsg=$repromptmsg;
		}
		$i++;
	}
	ThrowException("CQCLI_Invalid_Username");
}

sub AskForPassword
{
	my $cnt = $_[0];
	if( !($cnt) ){
		$cnt = 3;
	}
	my $i = 0;
	my $promptmsg = GetLocalizedMessage('CQCLI_Prompt_Password');
	my $repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_Password');
	my $password = undef;
	while($i<$cnt)
	{		
		CLIPrintOutput($promptmsg);
		#$password=<STDIN>;
		select((select(STDOUT), $| = 1)[0]);
		ReadMode 'noecho';
		$password= ReadLine 0;
		ReadMode 'restore';
		select((select(STDOUT), $| = 1)[0]);
		select((select(STDIN), $| = 1)[0]);
		CLIPrintOutput("\n");
		if(!($password)){
			return ;
		}
		chomp($password);
		$password=cqclitrim($password);
		# if(length $password){
			# return $password;
		# }
		# else{
			# $promptmsg=$repromptmsg;
		# }
		return $password;
		$i++;
	}
	ThrowException("CQCLI_Invalid_Password");
}

sub AskForDBName
{
	my $cnt = $_[0];
	if( !($cnt) ){
		$cnt = 3;
	}
	my $i = 0;
	my $promptmsg = GetLocalizedMessage('CQCLI_Prompt_Db');
	my $repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_Db');
	my $dbname = undef;
	while($i<$cnt)
	{
		CLIPrintOutput($promptmsg);
		$dbname=<STDIN>;
		if(!($dbname)){
			return ;
		}
		chomp($dbname);
		$dbname=cqclitrim($dbname);
		if(length $dbname){
			return $dbname;
		}
		else{
			$promptmsg=$repromptmsg;
		}
		$i++;
	}
	ThrowException("CQCLI_Invalid_Database");
}


sub AskForConnectionName
{
	my $cnt = $_[0];
	if( !($cnt) ){
		$cnt = 3;
	}
	my $i = 0;
	my $promptmsg = GetLocalizedMessage('CQCLI_Prompt_Connection');
	my $repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_Connection');
	my $connection = undef;
	while($i<$cnt)
	{
		
		CLIPrintOutput($promptmsg);
		$connection=<STDIN>;
		if(!($connection)){
			return ;
		}
		chomp($connection);
		$connection=cqclitrim($connection);
		if(length $connection){
			return $connection;
		}
		else{
			$promptmsg=$repromptmsg;
		}
		$i++;
	}
	ThrowException("CQCLI_Invalid_Connection");
}

sub AskForActionName
{
	my $action_list = $_[0];
	my $cnt =$_[1];
	if( !($cnt) ){
		$cnt = 3;
	}
	my $i = 0;
	$action_list =~s/\,/\n/g;
	$action_list = $action_list."\n";
	my $promptmsg = GetLocalizedMessage('CQCLI_Prompt_ActionNames',$action_list);
	my $repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_ActionNames',$action_list);
	my $action = undef;
	while($i<$cnt)
	{
		CLIPrintOutput($promptmsg);
		$action=<STDIN>;
		if(!($action)){
			return ;
		}
		chomp($action);
		$action = lc $action;
		$action=cqclitrim($action);
		if((length $action) && ($action_list =~/\s*\n?$action\s*\n/i)){
			return $action;
		}
		else{
			$promptmsg=$repromptmsg;
		}
		$i++;
	}
	ThrowException("CQCLI_Invalid_ActionName");
}

sub AskForFileName
{
	# my $file_name = $_[0];
	# my $cnt =$_[1];
	# if( !($cnt) ){
		# $cnt = 3;
	# }
	my $fromCommand = shift;
	my $i = 0;
	my $cnt = 3;
	# $file_name =~s/\,/\n/g;
	# $file_name = $file_name."\n";

	my $promptmsg = GetLocalizedMessage('CQCLI_Prompt_FileName');
	my $repromptmsg = $promptmsg;

	my $file = undef;
	while($i<$cnt)
	{
		CLIPrintOutput($promptmsg);
		$file=<STDIN>;
		if(!($file)){
			return ;
		}
		chomp($file);
		$file = lc $file;
		$file=cqclitrim($file);
		if((length $file)){
			return $file;
		}
		else{
			$promptmsg=$repromptmsg;
		}
		$i++;
	}
	
	if($fromCommand ne "remove_attachment") {
		if(!(-e $file ) ) {
				ThrowSyntaxErrorException("CQCLI_Missing_File",$file,undef);
		}
	}
}

sub AskForRecordId
{
	my $recid = $_[0];
	my $cnt = $_[1];
	my $promptmsg = undef;
	my ($repromptmsg,$rid) = (undef,undef);
	my $i = 0;

	if( !($cnt) ){
		$cnt = 3;
	}
	
	if($recid) {
		$promptmsg= GetLocalizedMessage('CQCLI_Prompt_Rid');
		$repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_Rid');
	}else{
		$promptmsg= GetLocalizedMessage('CQCLI_Prompt_Dbid');
		$repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_Dbid');
	}
	my $dbname = undef;
	while($i<$cnt)
	{
		CLIPrintOutput($promptmsg);
		$rid=<STDIN>;
		if(!($rid)){
			return ;
		}
		chomp($rid);
		$rid=cqclitrim($rid);
		if(length $rid){
			return $rid;
		}
		else{
			$promptmsg=$repromptmsg;
		}
		$i++;
	}
	ThrowException("CQCLI_Invalid_RId");
}

sub AskForDuplicateRecordId
{
	my $cnt = $_[0];
	my $promptmsg = undef;
	my ($repromptmsg,$rid) = (undef,undef);
	my $i = 0;

	if( !($cnt) ){
		$cnt = 3;
	}
	$promptmsg= GetLocalizedMessage('CQCLI_Prompt_DupRid');
	$repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_DupRid');
	my $dbname = undef;
	while($i<$cnt)
	{
		CLIPrintOutput($promptmsg);
		$rid=<STDIN>;
		if(!($rid)){
			return ;
		}
		chomp($rid);
		$rid=cqclitrim($rid);
		if(length $rid){
			return $rid;
		}
		else{
			$promptmsg=$repromptmsg;
		}
		$i++;
	}
	ThrowException("CQCLI_Invalid_RId");
}


sub cqclitrim($)
{
	my $string = shift;
	
	if(!($string)) {
		return $string;
	}

	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}
# Left trim function to remove leading whitespace
sub cqcliltrim($)
{
	my $string = shift;
	
	if(!($string)) {
		return $string;
	}
	
	$string =~ s/^\s+//;
	return $string;
}
# Right trim function to remove trailing whitespace
sub cqclirtrim($)
{
	my $string = shift;
	$string =~ s/\s+$//;
	return $string;
}


sub ctrl_c_handler  {
        print "\nExiting cqtool \n";
		exit 1;
}

sub AskForYN
{
	my $msgid = shift;
	my $msg = GetLocalizedMessage($msgid);
	my $flag = 1;
	my $yes = "no";
	while($flag)
	{
		CLIPrintOutput($msg);
		$yes = <STDIN>;
		if(!($yes)){
			return ;
		}
		chomp($yes);
		$yes = cqclitrim($yes);
		if ( !(length($yes)) ) {
			redo;
		}elsif( $yes =~ /^no?\b/i ) {
			return undef;
		}elsif(  $yes =~/^y(es)?\b/i ) {
			return 1;
		}
	}
	return undef;
}

sub WantToExit
{
	my $msgid = shift;
	my $param = shift;
	my $msg = GetLocalizedMessage($msgid,$param);
	my $flag = 1;
	my $yes = "no";
	while($flag)
	{
		CLIPrintOutput($msg);
		$yes = <STDIN>;
		chomp($yes);
		$yes = cqclitrim($yes);
		if ( !(length($yes)) ) {
			return undef;
		}elsif(  $yes =~/^y(es)?\b/i ) {
			return 1;
		}else{
			return undef;
		}
	}
	return undef;
}

sub CliJavaPrintOutput
{
	my $message = shift;
	cqcli::Util::CQCLIUtil->printString($message);
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
